/*
 * emulator_graphics.h
 *
 *  Created on: Oct 22, 2010
 *      Author: Squarepusher2
 */
#ifndef EMULATOR_GRAPHICS_H_
#define EMULATOR_GRAPHICS_H_

#include <sysutil/sysutil_sysparam.h>
 
#define EMU_DBG_DELAY 2000000

#ifdef EMUDEBUG
#include "cellgraphicspsgl.h"
	#define EMU_DBG(fmt, args...) do {\
	   gl_dprintf(0.1, 0.1, 1.0, fmt, ##args);\
	   sys_timer_usleep(EMU_DBG_DELAY);\
	   } while(0)
#else
#define EMU_DBG(fmt, args...) ((void)0)
#endif

//#define EMULATOR_ASPECT_RATIO_16_9 0;
//#define EMULATOR_ASPECT_RATIO_4_3 1;

#define SNES_RENDER_TEXTURE_WIDTH (512)
#define SNES_RENDER_TEXTURE_HEIGHT (512)

#define SCREEN_16_9_ASPECT_RATIO (16.0/9)
#define SCREEN_4_3_ASPECT_RATIO (4.0/3)
#define SNES_ASPECT_RATIO SCREEN_4_3_ASPECT_RATIO

#define SNES9X_SCREEN_PITCH (1024)

#define SCREEN_RENDER_TEXTURE_WIDTH SNES_RENDER_TEXTURE_WIDTH
#define SCREEN_RENDER_TEXTURE_HEIGHT SNES_RENDER_TEXTURE_HEIGHT
#define SCREEN_REAL_ASPECT_RATIO SNES_ASPECT_RATIO
#define SCREEN_RENDER_TEXTURE_PITCH SNES9X_SCREEN_PITCH

#include "cellframework/graphics/PSGLGraphics.h"

typedef struct _Vertex
{
	float x;
	float y;
	float z;
} Vertex;

typedef struct _TextureCoord
{
	float u;
	float v;
} TextureCoord;

typedef struct _Quad
{
	Vertex v1;
	Vertex v2;
	Vertex v3;
	Vertex v4;
	TextureCoord t1;
	TextureCoord t2;
	TextureCoord t3;
	TextureCoord t4;
} Quad;

extern uint8_t *vertex_buf;
extern Quad screenQuad;


void Graphics_ClearSurface();
void Graphics_InitDbgFont();
void Graphics_DeInitDbgFont();
void Graphics_RenderDbgFont();
void Graphics_SetViewports();
void Graphics_SetAspectRatio(bool keep_aspect);

// Compensate for TV overscan/underscan. e.g.: 4% overscan => amount = 0.04, 4% underscan => -0.04;
void Graphics_SetOverscan(bool overscan, float amount = 0.0);

int32_t Graphics_InitShader(const char* filename);
void Graphics_Render(bool sync);
void Graphics_SetTextureFilter(bool keep_smooth);
int Graphics_Init();
int Graphics_Init(uint32_t resId);
void Graphics_UpdateDimensions(unsigned width, unsigned height, unsigned tex_width, unsigned tex_height);
void Graphics_DeInit();
int32_t Graphics_ChangeResolution(uint32_t resId, uint16_t refreshrateId);
CellVideoOutState Graphics_GetVideoState();

#endif /* EMULATOR_GRAPHICS_H_ */
